(function(){'use strict';const FOCUSABLE_ELEMENTS = [
  '[href]:not([tabindex^="-"])',
  'input:not([disabled]):not([type="hidden"]):not([tabindex^="-"]):not([type="radio"])',
  'input[type="radio"]:checked',
  'select:not([disabled]):not([tabindex^="-"])',
  'textarea:not([disabled]):not([tabindex^="-"])',
  'button:not([disabled]):not([tabindex^="-"])',
  '[tabindex]:not([tabindex^="-"])',
  '[contenteditable="true"]:not([tabindex^="-"])',
];// Focus an element
// elements - array of dropdown elements
const focus =
    (elements, focussedIndex) => {
      // Use of % for infinite iterations
      var index = Math.abs(focussedIndex) % elements.length;
      if (index >= 0) {
        elements[index].focus();
      }
    };

// Refocus trigger element
const refocusTrigger =
    element => {
      var dialogTrigger = element.querySelector('[x-spread="trigger"]');
      if (dialogTrigger) {
        dialogTrigger.focus();
      }
    };

const adjustScrollBar =
    (isOpen) => {
      if (isOpen) {
        var htmlDocument = document.querySelector('html');
        htmlDocument.style.overflow = 'hidden';
        var body = document.querySelector('body');
        body.style.paddingRight = getScrollbarWidth() + 'px';
      } else {
        var htmlDocument = document.querySelector('html');
        htmlDocument.style.overflow = null;
        var body = document.querySelector('body');
        body.style.paddingRight = null;
      }
    };

// Private
// From bootstrap
const getScrollbarWidth =
    () => {
      const scrollDiv = document.createElement('div');
      scrollDiv.className = 'scrollbar-measure';
      document.body.appendChild(scrollDiv);
      const scrollbarWidth =
          scrollDiv.getBoundingClientRect().width - scrollDiv.clientWidth;
      document.body.removeChild(scrollDiv);
      return scrollbarWidth;
    };// Todo

// Data
var lastOpenedElement = null;

// Set attributes when the component is initialized
function init(dialog) {
  var trigger = dialog.querySelectorAll('[x-spread="trigger"]');
  if (trigger.length) {
    trigger = trigger[0];
    trigger.setAttribute('aria-haspopup', 'dialog');
    var dialog = dialog.querySelectorAll('[x-spread="dialog"]');
    if (dialog.length) {
      dialog = dialog[0];
      dialog.setAttribute('role', 'dialog');
      dialog.setAttribute('aria-hidden', true);
      dialog.setAttribute('aria-modal', true);
      dialog.setAttribute('tabindex', -1);
    }
  }
}

// Toggle aria atrributes based on the dialog state
function toggleAriaAtrributes(dialog, isOpen) {
  dialog = dialog.querySelectorAll('[x-spread="dialog"]');
  if (dialog.length) {
    dialog = dialog[0];
    if (isOpen) {
      dialog.setAttribute('aria-hidden', false);
    } else {
      dialog.setAttribute('aria-hidden', true);
    }
  }
}

// Initialize attribute for all dialog elements
var dialogs = document.querySelectorAll('[x-data="dialog()"]');
dialogs.forEach(function(dialog) {
  init(dialog);
});

window.dialog = function() {
  var focussedIndex = -1;
  return {
    open: false,
    trigger: {
      ['@click']() {
        this.open = !this.open;
        if (this.open) {
          lastOpenedElement = this.$el;
          var dialog = this.$el.querySelector('[x-spread="dialog"]');
          var dialogElements = dialog.querySelectorAll(FOCUSABLE_ELEMENTS);
          focussedIndex++;
          focus(dialogElements, focussedIndex);
        } else {
          focussedIndex = -1;
        }
        adjustScrollBar(this.open);
        toggleAriaAtrributes(this.$el, this.open);
      },
      ['@keydown.escape']() {
        this.open = false;
        focussedIndex = -1;
        refocusTrigger(lastOpenedElement);
        adjustScrollBar(this.open);
        toggleAriaAtrributes(this.$el, this.open);
      },
    },
    dialog: {
      ['x-show.transition.opacity.duration.100ms.origin.center.center.scale.105']() {
        return this.open;
      },
      ['@keydown.escape']() {
        this.open = false;
        focussedIndex = -1;
        refocusTrigger(lastOpenedElement);
        adjustScrollBar(this.open);
        toggleAriaAtrributes(this.$el, this.open);
      },
      ['@keydown.tab'](e) {
        e.preventDefault();
        var dialog = this.$el.querySelector('[x-spread="dialog"]');
        var dialogElements = dialog.querySelectorAll(FOCUSABLE_ELEMENTS);
        focussedIndex++;
        focus(dialogElements, focussedIndex);
      },
      ['@click'](e) {
        if (this.open && e.target.hasAttribute('x-spread')) {
          this.open = false;
          focussedIndex = -1;
          refocusTrigger(lastOpenedElement);
          adjustScrollBar(this.open);
          toggleAriaAtrributes(this.$el, this.open);
        }
      },
    },
    close() {
      this.open = false;
      focussedIndex = -1;
      refocusTrigger(lastOpenedElement);
      adjustScrollBar(this.open);
      toggleAriaAtrributes(this.$el, this.open);
    },
  };
};}());